#pragma once
#include "noexcept.h"	// swap, placement new
#pragma warning(disable:4786)	// Krzen der Bezeichnerlnge fr Debuginfo

// Ein std::vector-hnliches Konstrukt mit realloc()
// BUG: realloc() vertrgt sich nicht mit komplexen Datentypen, welche Zeiger und Referenzen enthalten!
// Eine durch realloc() verursachte Vektor-Verschiebung macht solche Zeiger ungltig: Ein generelles Problem.
// Eine Vektor-Vergrerung ruft Copy-Konstruktoren fr die Elemente auf, bei C++11 Move-Konstruktoren.
// Ich gehe im folgenden davon aus, dass der Kopierkonstruktor eine "flache Kopie" bzw. Raub-Kopie
// und der Zuweisungsoperator eine "tiefe Kopie" macht und sich dadurch wesentlich unterscheidet.
// Will man eine tiefe Kopie beim Konstruktor, dann mit einem zustzlichen (Dummy-bool-)Argument
// Will man eine flache (Raub-)Kopie bei einer Zuweisung, dann mit dem placement-new-Operator.
template<class T>class vector{
public:
 typedef T value_type;
 typedef value_type* iterator;
 typedef const value_type* const_iterator;
 typedef T& reference;
 typedef const T& const_reference;
 typedef size_t difference_type;
protected:
 iterator _begin,_end,_cap;
 static difference_type roundup(difference_type z) {return nextpow2(max<difference_type>(z,4));}
public:
 vector():_begin(0),_end(0),_cap(0) {}
 ~vector() {if (_begin) {clear(); free(_begin);}}
 explicit vector(difference_type N):_begin(new T[N]),_end(_begin+N),_cap(_begin+N) {}
// Normalerweise macht der Kopierkonstruktor und die Zuweisung eine tiefe Kopie.
// Da der Kopierkonstruktor bei jedem Array-Return auf den Plan gerufen wird,
// wrde das in einem wilden Herumkopieren ausarten,
// deshalb ist das hier ein Raubkopierkonstruktor.
// Fr den Fall dass "o" von __$ReturnUdt kommt muss alles bergangen werden
// Nicht den Zuweisungsoperator "=" benutzen!! Sondern memcpy. Und diesen inlinen.
 vector(const vector<T>&o) {
	if (this!=&o) {
	 memcpy(this,&o,sizeof*this);
	 memset(const_cast<vector<T>*>(&o),0,sizeof o);
	}
 }
 difference_type size() const{return _end-_begin;}
 difference_type capacity() const{return _cap-_begin;}
 bool reserve(difference_type cap) {
  if(cap>capacity()){
   cap=roundup(cap);
   difference_type siz=size();
   void*n=::realloc(_begin,cap*sizeof(T));
   if(!n)return false;
   _begin=(T*)n;
   _end=_begin+siz;
   _cap=_begin+cap;
  }return true;
 }
 bool commit_undefined(difference_type fill) {
  if(!reserve(fill))return false;
  _end=_begin+fill;
  return true;
 }
// bool push_back(const T*d,difference_type l) {
//  if(!reserve(_size+l))return false;memcpy(_data+_size,d,l*sizeof(T));_size+=l;return true;}
 bool push_back(const T&o) {return insert(_end,o);}
 void clear() {for(iterator it=_begin; it!=_end; it++) it->~T();_end=_begin;}
 vector<T>&operator=(const vector<T>&o) { // Tiefe Kopie
  if (this!=&o && reserve(o.size())) {	// Keine Zuweisung bei Objektgleichheit oder Speichermangel
   clear();	// Die vorhandenen Arrayelemente werden dem Destruktor unterzogen
   const_iterator s=o._begin;
   iterator d=_begin;	// Kein memcpy sondern jedes Element einzeln konstruieren
   for(; s!=o._end; s++,d++) new(d)T(*s);	// Placement-Copy-Konstruktor
   _end=d;		// Ende setzen
  }
  return*this;
 }
 const_reference operator[](difference_type i) const {return _begin[i];}
 reference operator[](difference_type i) {return _begin[i];}
 iterator begin() {return _begin;}
 const_iterator begin() const {return _begin;}
 iterator end() {return _end;}
 const_iterator end() const {return _end;}
 reference front() {return _begin[0];}
 const_reference front() const {return _begin[0];}
 reference back() {return _end[-1];}
 const_reference back() const {return _end[-1];}
 bool empty() const {return !size();}
 value_type*data() {return _begin;}
 const value_type*data() const {return _begin;}
 bool resize(difference_type siz) {if (!reserve(siz)) return false; _end=_begin+siz; return true;}

// insert and erase differ from std::vector as they check iterator and allocation,
// are noexcept and return false on failure, true on success, not an iterator
// Unsauber: Im Fall von T = uniq_ptr<> oder array_ptr<> wird el verndert, nmlich der Zeiger geraubt.
 bool insert(difference_type idx, difference_type elems, const T&el) {
  if (!elems) return true;
  if (idx>size()) return false;
  if(!reserve(size()+elems))return false;	// Iterator kann invalid werden!
  iterator it = _begin+idx;
  memmove(it+elems,it,(char*)_end-(char*)it);
  _end+=elems;
  do{
   new(it)T(const_cast<T&>(el));	// Placement-(Raub-)Kopier-Konstruktor, kein Zuweisungsoperator
   ++it;
  }while(--elems);
  return true;
 }
// Don't forget that iterator <it> is poison after calling insert()!
 bool insert(difference_type i,const T&el) {return insert(i,1,el);}
 bool insert(iterator it,difference_type elems, const T&el) {return insert(it-_begin,elems,el);}
 bool insert(iterator it,const T&el) {return insert(it,1,el);}

 bool erase(difference_type idx, difference_type elems=1) {
  if (!elems) return true;
  if (idx+elems>size()) return false;
  iterator it = _begin+idx;
  iterator start = it;	// Anfang zum Lschen merken
  _end-=elems;		// Ende ndern bevor elems zu Null iteriert wird
  do{
   it->~T();
   ++it;
  }while (--elems);
// now, it points to the first non-erased element, and _end beyond end of list.
  memmove(start,it,(char*)_end-(char*)it);
  return true;
 }
// Don't forget that iterator <it> may be poison after calling erase()!
 bool erase(iterator it,iterator ie) {return erase(it-_begin,ie-_begin);}
 bool erase(iterator it) {return erase(it-_begin);}
// using integer index is more handy in many cases,
// as they are not poison after calling these functions.
};

// For swapping, avoid the use of operator=.
// May use constructors in conjunction with placement-new!
template<class T>inline void swap(vector<T>&x,vector<T>&y) {
 typedef char vec[sizeof vector<T>];	// for copy of vector header
 vec t = *reinterpret_cast<vec*>(&x);	// swap the vector header, not the data
 *reinterpret_cast<vec*>(&x) = *reinterpret_cast<vec*>(&y);
 *reinterpret_cast<vec*>(&y) = t;
}
